@@ -1,7 +1,7 @@
 #' @title Generate text from text with Gemini
 #' @description Generate text from text with Gemini
 #' @param prompt The prompt to generate text from
-#' @param model The model to use. Options are "2.0-flash", "2.0-flash-lite", "2.5-pro-exp-03-25". Default is '2.0-flash'.
+#' @param model The model to use. Default is '2.0-flash'.
 #'              see https://ai.google.dev/gemini-api/docs/models/gemini
 #' @param temperature The temperature to use. Default is 1 value should be between 0 and 2
 #'              see https://ai.google.dev/gemini-api/docs/models/generative-models#model-parameters
@@ -13,7 +13,7 @@
 #'              see https://ai.google.dev/gemini-api/docs/models/generative-models#model-parameters
 #' @param maxOutputTokens The maximum number of tokens to generate.
 #'              Default is 8192 and 100 tokens correspond to roughly 60-80 words.
-#' @return Generated text
+#' @return Generated text or image
 #' @export
 #' @examples
 #' \dontrun{
@@ -22,61 +22,19 @@
 #' gemini("Explain dplyr's mutate function")
 #' }
 #' @importFrom httr2 request req_url_query req_headers req_body_json req_perform resp_body_json
-#' @importFrom cli cli_alert_danger cli_status_clear cli_status
+#' @importFrom cli cli_status_clear cli_status
 #'
 #' @seealso https://ai.google.dev/docs/gemini_api_overview#text_input
 #'
 
 gemini <- function(prompt, model = "2.0-flash", temperature = 1, maxOutputTokens = 8192, topK = 40, topP = 0.95, seed = 1234) {
-  if (is.null(prompt)) {
-    cli_alert_danger("{.arg prompt} must not NULL")
-    return(NULL)
-  }
-
-  if (!is.character(prompt)) {
-    cli_alert_danger("{.arg prompt} must be given as a STRING")
-    return(NULL)
-  }
-
-  # API Key
-  if (Sys.getenv("GEMINI_API_KEY") == "") {
-    cli_alert_danger("Please set the {.envvar GEMINI_API_KEY} with {.fn setAPI} function.")
-    return(NULL)
-  }
-
-  # Model
-  supported_models <- c("2.0-flash", "2.0-flash-lite", "2.5-pro-exp-03-25", "2.0-flash-exp-image-generation")
-
-  if (!(model %in% supported_models)) {
-    cli_alert_danger("Error: Parameter 'model' must be one of '2.0-flash', '2.0-flash-lite', '2.5-pro-exp-03-25', '2.0-flash-exp-image-generation'")
+  # Validate all parameters at once
+  if (!validate_params(prompt, model, temperature, topP, topK, seed, api_key = TRUE)) {
     return(NULL)
   }
 
   model_query <- paste0("gemini-", model, ":generateContent")
-
-  # Parameters
-  if (temperature < 0 | temperature > 2) {
-    cli_alert_danger("Error: Parameter 'temperature' must be between 0 and 2")
-    return(NULL)
-  }
-
-  if (topP < 0 | topP > 1) {
-    cli_alert_danger("Error: Parameter 'topP' must be between 0 and 1")
-    return(NULL)
-  }
-
-  if (topK < 0 | topK > 100) {
-    cli_alert_danger("Error: Parameter 'topK' must be between 0 and 100")
-    return(NULL)
-  }
-
-  if (!is.numeric(seed) || seed %% 1 != 0) {
-    cli_alert_danger("Error: Parameter 'seed' must be an integer")
-    return(NULL)
-  }
-
   url <- paste0("https://generativelanguage.googleapis.com/v1beta/models/", model_query)
-
   api_key <- Sys.getenv("GEMINI_API_KEY")
 
   sb <- cli_status("Gemini is answering...")
@@ -94,25 +52,34 @@ gemini <- function(prompt, model = "2.0-flash", temperature = 1, maxOutputTokens
   if (model == "2.0-flash-exp-image-generation") {
     generation_config$responseModalities <- list("Text", "Image")
   }
+  
+  # Create request body as a separate list
+  request_body <- list(
+    contents = list(
+      parts = list(
+        list(text = prompt)
+      )
+    ),
+    generationConfig = generation_config
+  )
 
   req <- request(url) |>
     req_url_query(key = api_key) |>
     req_headers("Content-Type" = "application/json") |>
-    req_body_json(list(
-      contents = list(
-        parts = list(
-          list(text = prompt)
-        )
-      ),
-      generationConfig = generation_config
-    ))
+    req_body_json(request_body)
   resp <- req_perform(req)
+  
+  # 상태 코드 검증 로직 추가
+  if (resp$status_code != 200) {
+    cli_status_clear(id = sb)
+    cli_alert_danger(paste0("Error in generate request: Status code ", resp$status_code))
+    return(NULL)
+  }
 
   cli_status_clear(id = sb)
 
   candidates <- resp_body_json(resp)$candidates
   outputs <- unlist(lapply(candidates, function(candidate) candidate$content$parts))
-
   return(outputs)
 }
 
@@ -147,42 +114,22 @@ gemini <- function(prompt, model = "2.0-flash", temperature = 1, maxOutputTokens
 
 gemini.vertex <- function(prompt = NULL, tokens = NULL, temperature = 1, maxOutputTokens = 8192,
                           topK = 40, topP = 0.95, seed = 1234) {
-  if (is.null(prompt)) {
-    cli_alert_danger("{.arg prompt} must not NULL")
+  if (!validate_params(prompt, NULL, temperature, topP, topK, seed, api_key = FALSE, tokens = tokens)) {
     return(NULL)
   }
 
-  if (!is.character(prompt)) {
-    cli_alert_danger("{.arg prompt} must be given as a STRING")
-    return(NULL)
-  }
-
-  if (is.null(tokens)) {
-    cli_alert_danger("{.arg tokens} must not be NULL. Use token.vertex() function to generate tokens.")
-    return(NULL)
-  }
-
-  # Parameters validation
-  if (temperature < 0 | temperature > 2) {
-    cli_alert_danger("Error: Parameter 'temperature' must be between 0 and 2")
-    return(NULL)
-  }
-
-  if (topP < 0 | topP > 1) {
-    cli_alert_danger("Error: Parameter 'topP' must be between 0 and 1")
-    return(NULL)
-  }
-
-  if (topK < 0 | topK > 100) {
-    cli_alert_danger("Error: Parameter 'topK' must be between 0 and 100")
-    return(NULL)
-  }
-
-  if (!is.numeric(seed) || seed %% 1 != 0) {
-    cli_alert_danger("Error: Parameter 'seed' must be an integer")
-    return(NULL)
-  }
+  sb <- cli_status("Gemini Vertex is answering...")
 
+  # Create generation config
+  generation_config <- list(
+    temperature = temperature,
+    maxOutputTokens = maxOutputTokens,
+    topP = topP,
+    topK = topK,
+    seed = seed
+  )
+  
+  # Create request body as a separate list
   request_body <- list(
     contents = list(
       list(
@@ -192,27 +139,32 @@ gemini.vertex <- function(prompt = NULL, tokens = NULL, temperature = 1, maxOutp
         )
       )
     ),
-    generationConfig = list(
-      temperature = temperature,
-      maxOutputTokens = maxOutputTokens,
-      topP = topP,
-      topK = topK,
-      seed = seed
-    )
+    generationConfig = generation_config
   )
 
-  sb <- cli_status("Gemini is answering...")
-
-  response <- request(tokens$url) |>
+  # API 요청 부분을 분리하여 상태 코드 검증
+  req <- request(tokens$url) |>
     req_headers(
       "Authorization" = paste0("Bearer ", tokens$key),
       "Content-Type" = "application/json"
     ) |>
-    req_body_json(request_body) |>
-    req_perform() |>
-    resp_body_json()
+    req_body_json(request_body)
+  
+  resp <- req_perform(req)
+  
+  # 상태 코드 검증 로직 추가
+  if (resp$status_code != 200) {
+    cli_status_clear(id = sb)
+    cli_alert_danger(paste0("Error in generate request: Status code ", resp$status_code))
+    return(NULL)
+  }
+  
+  response <- resp_body_json(resp)
 
   cli_status_clear(id = sb)
-
-  return(response$candidates[[1]]$content$parts[[1]]$text)
+  
+  candidates <- response$candidates
+  outputs <- unlist(lapply(candidates, function(candidate) candidate$content$parts))
+  
+  return(outputs)
 }
