@@ -38,82 +38,60 @@
 
 gemini_chat <- function(prompt, history = list(), model = "2.0-flash", temperature = 1,
                         maxOutputTokens = 8192, topK = 40, topP = 0.95, seed = 1234) {
-  if (is.null(prompt)) {
-    cli_alert_danger("{.arg prompt} must not NULL")
-    return(NULL)
-  }
-
-  if (!is.character(prompt)) {
-    cli_alert_danger("{.arg prompt} must be given as a STRING")
-    return(NULL)
-  }
-
-  if (Sys.getenv("GEMINI_API_KEY") == "") {
-    cli_alert_danger("Please set the {.envvar GEMINI_API_KEY} with {.fn setAPI} function.")
-    return(NULL)
-  }
-
-  # Model
-  supported_models <- c("2.0-flash", "2.0-flash-lite", "2.5-pro-exp-03-25")
-
-  if (!(model %in% supported_models)) {
-    cli_alert_danger("Error: Parameter 'model' must be one of '2.0-flash', '2.0-flash-lite', '2.5-pro-exp-03-25'")
-    return(NULL)
-  }
-
-
-  model_query <- paste0("gemini-", model, ":generateContent")
-
-
-  if (temperature < 0 | temperature > 2) {
-    cli_alert_danger("Error: Parameter 'temperature' must be between 0 and 2")
-    return(NULL)
-  }
-
-  if (topP < 0 | topP > 1) {
-    cli_alert_danger("Error: Parameter 'topP' must be between 0 and 1")
-    return(NULL)
-  }
-
-  if (topK < 0 | topK > 100) {
-    cli_alert_danger("Error: Parameter 'topK' must be between 0 and 100")
-    return(NULL)
-  }
-
-  if (!is.numeric(seed) || seed %% 1 != 0) {
-    cli_alert_danger("Error: Parameter 'seed' must be an integer")
+  # 1. validate_params 함수 사용하여 파라미터 검증
+  if (!validate_params(prompt, model, temperature, topP, topK, seed, api_key = TRUE)) {
     return(NULL)
   }
 
   history <- history |>
     addHistory(role = "user", item = prompt)
 
+  model_query <- paste0("gemini-", model, ":generateContent")
   url <- paste0("https://generativelanguage.googleapis.com/v1beta/models/", model_query)
   api_key <- Sys.getenv("GEMINI_API_KEY")
 
   sb <- cli_status("Gemini is answering...")
+  
+  # 2. generation_config를 별도 리스트로 구성
+  generation_config <- list(
+    temperature = temperature,
+    maxOutputTokens = maxOutputTokens,
+    topP = topP,
+    topK = topK,
+    seed = seed
+  )
+  
+  # 요청 본문을 별도의 변수로 생성
+  request_body <- list(
+    contents = history,
+    generationConfig = generation_config
+  )
+  
   req <- request(url) |>
     req_url_query(key = api_key) |>
     req_headers("Content-Type" = "application/json") |>
-    req_body_json(list(
-      contents = history,
-      generationConfig = list(
-        temperature = temperature,
-        maxOutputTokens = maxOutputTokens,
-        topP = topP,
-        topK = topK,
-        seed = seed
-      )
-    ))
+    req_body_json(request_body)
 
   resp <- req_perform(req)
+  
+  # 3. 상태 코드 검증 추가
+  if (resp$status_code != 200) {
+    cli_status_clear(id = sb)
+    cli_alert_danger(paste0("Error in generate request: Status code ", resp$status_code))
+    return(NULL)
+  }
+  
   cli_status_clear(id = sb)
 
   candidates <- resp_body_json(resp)$candidates
   outputs <- unlist(lapply(candidates, function(candidate) candidate$content$parts))
-
-  history <- history |>
-    addHistory(role = "model", item = outputs[[1]])
+  
+  # 4. 응답 처리 개선 - 여러 응답을 처리할 수 있도록 수정
+  if (length(outputs) > 0) {
+    # 첫 번째 응답을 기록
+    history <- history |>
+      addHistory(role = "model", item = outputs[[1]])
+  }
 
   return(list(outputs = outputs, history = history))
 }
@@ -124,17 +102,28 @@ gemini_chat <- function(prompt, history = list(), model = "2.0-flash", temperatu
 #' @param role The role of chat: "user" or "model"
 #' @param item The item of chat: "prompt" or "output"
 #' @return The history of chat
+#' @keywords internal
+#' @importFrom cli cli_alert_danger
 #'
 
 addHistory <- function(history, role = NULL, item = NULL) {
+  # 5. addHistory 함수 개선 - 오류 메시지 개선
   if (is.null(role)) {
-    cli_alert_danger("provide role")
+    cli_alert_danger("{.arg role} must not be NULL")
     return(NULL)
   }
   if (is.null(item)) {
-    cli_alert_danger("provide item")
+    cli_alert_danger("{.arg item} must not be NULL")
     return(NULL)
   }
+  
+  # 역할 검증 추가
+  valid_roles <- c("user", "model")
+  if (!(role %in% valid_roles)) {
+    cli_alert_danger(paste0("Invalid role: '", role, "'. Must be one of: ", paste(valid_roles, collapse=", ")))
+    return(NULL)
+  }
+  
   history[[length(history) + 1]] <-
     list(
       role = role,
